<?php
// $Id: uts.analysis.inc,v 1.13 2008/07/13 06:26:05 boombatower Exp $
/**
 * @file
 * Provide data analysis pages for the Usability Testing Suite.
 *
 * Copyright 2008 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Menu callback: Display list of studies to analyze, list of sessions to
 * analyze, display particular plugin generated analysis, or perform operation
 * on plug-in.
 *
 * @param interger $study_nid Current study NID.
 * @return string HTML Output.
 */
function uts_analysis($study_nid = NULL) {
  if ($study_nid) {
    // Study select display sessions or display plugin related material.
    $sessions = uts_session_load_all($study_nid);
    $analysis_plugins = uts_data_analysis_list();

    // Parse the remaining path arguments.
    $args = uts_analysis_get_args($study_nid, $sessions, $analysis_plugins);

    // Set title to include study and session id if set.
    $study = node_load($study_nid);
    if ($args['session_id']) {
      drupal_set_title(t('Analyze %study: %session', array('%study' => $study->title, '%session' => $args['session_id'])));
    }
    else {
      drupal_set_title(t('Analyze %study', array('%study' => $study->title)));
    }

    // Add local tasks for plug-ins and modes.
    uts_analysis_local_tasks_add($study_nid, $args['session_id'], $args['plugin'], $analysis_plugins);

    // If plug-in sepected either perform selcted action or display data.
    if ($args['plugin']) {
      // Get timestamp range.
      if ($args['session_id']) {
        list($start, $stop) = uts_data_get_timestamps($args['session_id']); // TODO Provide timestamp filter.
      }
      else {
        list($start, $stop) = array(NULL, NULL);
      }

      // If action specified then perform action instead of displaying analysis.
      if ($args['action'] == 'export') {
        uts_analysis_export($args['plugin'], $args['mode'], $study_nid, $args['session_id'], $start, $stop);
        exit();
      }

      // Display data analysis from plug-in.
      $output = uts_analysis_display($args['plugin'], $args['mode'], $study_nid, $args['session_id'], $start, $stop);

      // Add action links.
      if (!$args['action']) {
        $output .= l(t('Export'), $_GET['q'] . '/export');
      }
      return $output;
    }

    // Display table of sessions related to study.
    $headers = array(t('Name'), t('Status'), t('Total time'), t('Operations'));
    $rows = array();
    foreach ($sessions as $session_id) {
      $row = array();
      $session = uts_session_load($session_id);

      $row[] = $session->session_id;
      $row[] = ($session->complete ? t('complete') : t('incomplete'));
      $row[] = format_interval(uts_data_get_total_time($session->session_id));
      $row[] = uts_render_operations(uts_session_operations($session));
      $rows[] = $row;
    }
    return t('<p>Use the analysis tabs to view overal study analysis, or select an individual session for analysis.</p>
              !sessions', array('!sessions' => theme('table', $headers, $rows)));
  }
  else {
    // No study selected, display list of active and closed studies.
    $studies = array_merge(uts_studies_load(UTS_STUDY_STATUS_ACTIVE), uts_studies_load(UTS_STUDY_STATUS_CLOSED));
    $list = array();
    foreach ($studies as $study) {
      $list[] = t('!title<br />@description',
                   array('!title' => l($study->title, 'admin/uts/analyze/' . $study->nid), '@description' => t($study->body)));
    }
    return t('<p>Select a study to analyze.</p>!studies', array('!studies' => theme('item_list', $list)));
  }
}

/**
 * Get the display elements from the analysis plug-ins, sort them by weight,
 * and collect the output.
 *
 * @param string $module Data colleciton module from which data will be retreived.
 * @param string $mode The analysis mode.
 * @param interger $study_nid Study NID.
 * @param string $session_id Session ID.
 * @param interger $start_timestamp Start timestamp.
 * @param integer $stop_timestamp Stop timestamp.
 * @return string HTML output.
 */
function uts_analysis_display($plugin, $mode, $study_nid, $session_id, $start_timestamp, $stop_timestamp) {
  $elements = uts_data_analysis_display($plugin, $mode, $study_nid, $session_id, $start_timestamp, $stop_timestamp);
  uasort($elements, 'element_sort');

  // Output all the elements which have been sorted by weight.
  $output = '';
  foreach ($elements as $element) {
    $output .= $element['#data'];
  }
  return $output;
}

/**
 * Collection export files from analysis plug-ins, create ZIP archive, and
 * output to browser.
 *
 * @param string $module Data colleciton module from which data will be retreived.
 * @param string $mode The analysis mode.
 * @param interger $study_nid Study NID.
 * @param string $session_id Session ID.
 * @param interger $start_timestamp Start timestamp.
 * @param integer $stop_timestamp Stop timestamp.
 */
function uts_analysis_export($plugin, $mode, $study_nid, $session_id, $start_timestamp, $stop_timestamp) {
  require_once drupal_get_path('module', 'uts') . '/uts.zip.inc';

  // Get data and generate ZIP file.
  $exports = uts_data_analysis_export($plugin, $mode, $study_nid, $session_id, $start_timestamp, $stop_timestamp);
  $zip = new ZipFile();
  foreach ($exports as $file => $data) {
    $zip->addFile($data, $file);
  }

  // Output file.
  header("Content-Disposition: attachment; filename=\"$plugin-$mode.zip\"");
  header("Content-Type: application/zip");
  header("Content-Transfer-Encoding: binary");

  echo $zip->file();
}

/**
 * Parse the path arguments that define the analysis operation.
 *
 * @param interger $study_nid Current study NID.
 * @param array $sessions List of session IDs.
 * @param array $analysis_plugins List of analysis plug-ins.
 * @return array Parsed arguments.
 */
function uts_analysis_get_args($study_nid, $sessions, $analysis_plugins) {
  $args = array(
    'session_id' => NULL,
    'plugin' => NULL,
    'mode' => NULL,
    'action' => NULL
  );

  // Url will either be in format session_id/plugin/mode/action, or plugin/mode/action.
  if (in_array(arg(4), $sessions)) {
    // URL format: session_id/plugin/mode/action.
    list($args['session_id'], $args['plugin'], $args['mode'], $args['action']) = array(arg(4), arg(5), arg(6), arg(7));
  }
  else {
    // URL format: plugin/mode/action.
    list($args['plugin'], $args['mode'], $args['action']) = array(arg(4), arg(5), arg(6));
  }

  // Clean values.
  if (!array_key_exists($args['plugin'], $analysis_plugins)) {
    $args['plugin'] = NULL;
  }
  if ($args['mode']) {
    $plugin_modes = $analysis_plugins[$args['plugin']];
    if (!in_array($args['mode'], $plugin_modes)) {
      $args['mode'] = NULL;
    }
  }
  if (!in_array($args['action'], array('export'))) {
    $args['action'] = NULL;
  }
  return $args;
}

/**
 * Add local tasks for analysis.
 *
 * @param interger $study_nid Current study NID.
 * @param string $session_id Current sessiond ID.
 * @param string $plugin Current data analysis plug-in.
 * @param array $analysis_plugins List a analysis plug-ins and modes.
 */
function uts_analysis_local_tasks_add($study_nid, $session_id, $plugin, $analysis_plugins) {
  if ($study_nid) {
    uts_local_tasks(0, t('Sessions'), "admin/uts/analyze/$study_nid/sessions", !$plugin);

    // Display plug-in links.
    $base_url = "admin/uts/analyze/$study_nid" . ($session_id ? "/$session_id" : '');
    $collection_plugins = uts_data_collection_list();
    foreach ($analysis_plugins as $module => $modes) {
      uts_local_tasks(0, $collection_plugins[$module]['name'], "$base_url/$module/{$modes[0]}", $module == $plugin);
    }

    // If plug-in selected display mode links.
    if ($plugin) {
      $base_url .= "/$plugin";
      $plugin_modes = $analysis_plugins[$plugin];
      if (count($plugin_modes) > 1) {
        // Display modes.
        foreach ($plugin_modes as $plugin_mode) {
          uts_local_tasks(1, t($plugin_mode), "$base_url/$plugin_mode", !$plugin);
        }
      }
    }
  }
}

/**
 * Get an array containing the session operation links.
 *
 * @param $session Session object.
 * @return Array of links.
 */
function uts_session_operations($session) {
  $links = array();
  if ($session->complete) {
    $links['analyze'] = array(
      'title' => t('analyze'),
      'href' => "admin/uts/analyze/$session->study_nid/$session->session_id"
    );
  }
  $links['session_destroy'] = array(
    'title' => t('destroy'),
    'href' => "admin/uts/analyze/$session->study_nid/$session->session_id/destroy"
  );
  return $links;
}

/**
 * Menu callback -- ask for confirmation of session destruction then redirect
 * to appropriate overview page.
 */
function uts_analysis_destroy_confirm(&$form_state, $session_id) {
  $redirect = 'admin/uts/analyze/' . arg(3);

  // Inject form values.
  $form['session_id'] = array(
    '#type' => 'value',
    '#value' => $session_id,
  );
  $form['redirect'] = array(
    '#type' => 'value',
    '#value' => $redirect,
  );

  return confirm_form($form,
    t('Are you sure you want to delete %session?', array('%session' => $session_id)),
    isset($_GET['destination']) ? $_GET['destination'] : $redirect,
    t('This action cannot be undone.'),
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Execute session destruction.
 */
function uts_analysis_destroy_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    uts_session_destory($form_state['values']['session_id']);
  }

  // Set the redirect to the pre-determined location.
  $form_state['redirect'] = $form_state['values']['redirect'];
}
