<?php
// $Id: uts.environment.inc,v 1.5 2008/07/03 05:42:26 boombatower Exp $
/**
 * @file
 * Provide Usability Testing Suite testing environment functions.
 *
 * Copyright 2008 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Invoke the callback function in another process using a http request. This
 * ensures that static variables that are messed up during environment create
 * and destroy do no effect the user experience.
 *
 * @param string $op Operation to perform (create, destroy).
 * @param string $prefix Environment prefix to perform opperation on.
 * @param string $second If performing create opperation an environment
 *   be used as a base to copy from. If performing enabling or disabling
 *   modules then a comman separate list of modules can be specified.
 * @return boolean Success.
 */
function uts_environment_invoke_callback($op, $prefix, $second = NULL) {
  // Set internal request data for checking in other thread.
  variable_set('uts_environment', "$op:$prefix:$second");

  $headers = array('User-Agent' => 'UTS-Environment-Callback');
  $url = url("uts/environment/$op/$prefix/$second", array('absolute' => TRUE));
  $result = drupal_http_request($url, $headers);

  // Fix any issues after creating environment.
  uts_environment_rebuild_caches();

  if ($result->data != 'success') {
    return FALSE;
  }

  return TRUE;
}

/**
 * Menu callback that allows environment opperation to be called
 * in a seperate process via a http request.
 *
 * @param string $op Operation to perform (create, destroy).
 * @param string $prefix Environment prefix to perform opperation on.
 * @param string $second If performing create opperation an environment
 *   be used as a base to copy from. If performing enabling or disabling
 *   modules then a comman separate list of modules can be specified.
 * @return Output status code.
 */
function uts_environment_callback($op, $prefix, $second = NULL) {
  if (variable_get('uts_environment', '') != "$op:$prefix:$second" ||
      $_SERVER['HTTP_USER_AGENT'] != 'UTS-Environment-Callback') {
    // Invalid request.
    exit('error');
  }
  variable_del('uts_environment');

  if ($op == 'create') {
    uts_environment_create($prefix, $second);
  }
  else if ($op == 'destroy') {
    uts_environment_destroy($prefix);
  }
  else if ($op == 'enable_modules') {
    uts_environment_modules($prefix, $second, TRUE);
  }
  else if ($op == 'disable_modules') {
    uts_environment_modules($prefix, $second, FALSE);
  }
  else {
    exit('error');
  }
  exit('success');
}

/**
 * Create prefixed environment, including database and files directory.
 *
 * @param string $prefix Prefix to use for new environment.
 * @param string $copy_prefix Prefix of environment to copy.
 */
function uts_environment_create($prefix, $copy_prefix = NULL) {
  global $db_prefix;

  // Get max nid, and vid for use below.
  $result = db_fetch_object(db_query('SELECT MAX(nid) AS max_nid, MAX(vid) AS max_vid FROM {node}'));
  $max_nid = $result->max_nid;
  $max_vid = $result->max_vid;

  // Store necessary current values before switching to prefixed database.
  $db_prefix_original = $db_prefix;
  $clean_url_original = variable_get('clean_url', 0);

  // Generate temporary prefixed database to ensure that test sessions have a clean starting point.
  $db_prefix = $prefix;
  include_once './includes/install.inc';
  drupal_install_system();

  $modules = drupal_verify_profile('default', 'en');

  // Add required UTS modules.
  $modules[] = 'uts_proctor';

  if ($copy_prefix) {
    // TODO Load modules from study environment.
    $modules = array();
  }
  drupal_install_modules($modules);

  // Run default profile tasks.
  $task = 'profile';
  default_profile_tasks($task, '');

  // Rebuild caches.
  uts_environment_rebuild_caches();

  // Restore necessary variables.
  variable_set('install_profile', 'default');
  variable_set('install_task', 'profile-finished');
  variable_set('clean_url', $clean_url_original);

  // Use temporary files directory with the same prefix as database.
  variable_set('file_directory_path', file_directory_path() . '/' . $prefix);
  file_check_directory(file_directory_path(), TRUE); // Create the files directory.

  if ($copy_prefix) {
    // TODO Copy any files and override database.
  }

  // Add required UTS variables.
  variable_set('uts_db_prefix', $db_prefix_original);

  // Insert placeholder node to alliviate any node_load issues with
  // reading form UTS database.
  $nid = $max_nid + 1000;
  $vid = $max_vid + 1000;
  $title = 'Place-holder';
  $body = "-- $title --";
  db_query("INSERT INTO {node}
            (nid, vid, type, title)
            VALUES (%d, %d, '%s', '%s')", $nid, $vid, 'page', $title);
  db_query("INSERT INTO {node_revisions}
            (nid, vid, title, body, teaser, log)
            VALUES (%d, %d, '%s', '%s', '%s', '%s')", $nid, $vid, $title, $body, $body, '');

  // Restore original environment.
  $db_prefix = $db_prefix_original;
  uts_environment_rebuild_caches();
}

/**
 * Destroy prefixed environment.
 *
 * @param string $prefix Prefix of environment to destroy.
 */
function uts_environment_destroy($prefix) {
  global $db_prefix;

  // Store necessary current values before switching to prefixed database.
  $db_prefix_original = $db_prefix;
  $clean_url_original = variable_get('clean_url', 0);

  // Switch to environment to destroy.
  $db_prefix = $prefix;
  uts_environment_rebuild_caches();

  // Delete temporary files directory and reset files directory path.
  uts_environment_destroy_directory(file_directory_path());

  // Remove all prefixed tables (all the tables in the schema).
  $schema = drupal_get_schema(NULL, TRUE);
  $ret = array();
  foreach ($schema as $name => $table) {
    db_drop_table($ret, $name);
  }

  // Restore original environment.
  $db_prefix = $db_prefix_original;
  uts_environment_rebuild_caches();
}

/**
 * Perform operations on modules, either enable or disable. Executes
 * system_modules form to ensure that all caches are cleared properly.
 *
 * @param string $prefix Prefix of environment to enable modules in.
 * @param string $modules Comma separated list of modules.
 * @param boolean $enable Enable/Disable list of modules.
 */
function uts_environment_modules($prefix, $modules, $enable) {
  global $db_prefix;

  // Switch database.
  $db_prefix_original = $db_prefix;
  $db_prefix = $prefix;
  uts_environment_rebuild_caches();

  // Prepare form_state array.
  $modules = explode(',', $modules);
  $form_state = array();
  foreach ($modules as $module) {
    $form_state['values']["status[$module]"] = $enable;
  }

  $reserved_modules = module_list(TRUE, FALSE);
  foreach ($reserved_modules as $module) {
    $form_state['values']["status[$module]"] = TRUE;
  }

  // Execute system_modules form to ensure that all caches are cleared properly.
  require_once drupal_get_path('module', 'system') . '/system.admin.inc';
  drupal_execute('system_modules', $form_state);

  // Changes don't get saved so force them.
  if ($enable) {
    module_enable($modules);
  }
  else {
    module_disable($modules);
  }
  uts_environment_rebuild_caches();

  // Restore original environment.
  $db_prefix = $db_prefix_original;
  uts_environment_rebuild_caches();
}

/**
 * Refresh all caches to ensure that static variables reflect current environment.
 */
function uts_environment_rebuild_caches() {
  global $conf;
  cache_clear_all('variables', 'cache');
  $conf = variable_init();

  module_list(TRUE, FALSE);
  module_rebuild_cache();
  drupal_flush_all_caches();
  menu_rebuild();

  actions_synchronize();
}

/**
 * Remove all files from specified directory and then remove directory.
 *
 * @param string $path Directory path.
 */
function uts_environment_destroy_directory($path) {
  $files = scandir($path);
  foreach ($files as $file) {
    if ($file != '.' && $file != '..') {
      $file_path = "$path/$file";
      if (is_dir($file_path)) {
        uts_environment_destroy_directory($file_path);
      }
      else {
        file_delete($file_path);
      }
    }
  }
  rmdir($path);
}
